/**
 * @file
 * Site preview.
 */

(function($, window, Drupal, drupalSettings) {
  $.fn.drupalAjax = (url) => {
    const ajaxSettings = {
      url,
      dialogRenderer: 'off_canvas',
      dialogType: 'dialog',
      // @todo make width configurable and pass via drupalSettings
      dialog: { width: 440 },

    };

    Drupal.ajax(ajaxSettings).execute();

    return;
  }

  $.fn.iframePreviewReload = (storage, uuid) => {
    const { framework } = drupalSettings.composable_editor.iframe_preview;
    const { selector, src, node } = drupalSettings[framework].iframe_preview;
    const $iframe = $(selector);

    if (storage === 'paragraph' && uuid) {
      $iframe[0].src = `${src}?${new Date().getTime()}#paragraph-${uuid}`;
      $.fn.drupalAjax(`/composable_editor/form/node/${node}/edit`);
    }

    if (storage === 'node' && uuid) {
      $iframe[0].src = src;
      // Reload node after save to refresh form state.
      $.fn.drupalAjax(`/composable_editor/form/node/${uuid}/edit`);
    }

    return;
  }

  Drupal.behaviors.iframeParagraphActions = {
    attach() {
      const { framework } = drupalSettings.composable_editor.iframe_preview;
      const { node } = drupalSettings[framework].iframe_preview;
      jQuery('#drupal-off-canvas .paragraphs-actions input[data-composable-uuid]').each(function(index, element) {
        jQuery(element).click(function(event) {
          event.preventDefault();

          const uuid = jQuery(event.target).data('composable-uuid');
          const storage = jQuery(event.target).data('composable-storage');
          const action = jQuery(event.target).data('composable-action');

          $.fn.drupalAjax(`/composable_editor_paragraphs/form/node/${node}/${storage}/${uuid}/${action}`);

          return;
        });
      });

      jQuery('#drupal-off-canvas .form-actions input.field-add-more-submit').each(function(index, element) {
        jQuery(element).click(function(event) {
          event.preventDefault();

          const uuid = jQuery(event.target).data('composable-uuid');
          const storage = jQuery(event.target).data('composable-storage');
          const action = jQuery(event.target).data('composable-action');
          const bundle = jQuery(event.target).data('composable-bundle');
          const field = jQuery(event.target).data('composable-field');

          $.fn.drupalAjax(`/composable_editor_paragraphs/form/node/${uuid}/${storage}/${action}/${bundle}/${field}`);

          return;
        });
      });

    }
  };

  Drupal.behaviors.iframePreviewLoader = {
    attach(context) {
      const { framework, open_load } = drupalSettings.composable_editor.iframe_preview;
      const { selector, node } = drupalSettings[framework].iframe_preview;
      once('composable_preview_iframe', selector, context).forEach(function () {
        if (open_load && node) {
          $.fn.drupalAjax(`/composable_editor/form/node/${node}/edit`);
        }
      });
    }
  };

  Drupal.behaviors.iframePreviewComposable = {
    attach() {
      const { framework } = drupalSettings.composable_editor.iframe_preview;
      const { node } = drupalSettings[framework].iframe_preview;
      window.addEventListener("message", (event) => {
        const { data } = event
        if (data.type === 'DRUPAL_COMPOSABLE_COMPONENT') {
          const { uuid, storage, action } = data;
          $.fn.drupalAjax(`/composable_editor_paragraphs/form/node/${node}/${storage}/${uuid}/${action}`);
          event.stopImmediatePropagation();

          return;
        }
        
      }, false);
    }
  }

})(jQuery, window, Drupal, drupalSettings);
