<?php

namespace Drupal\composable_editor\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityRepository;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Controller to edit Paragraphs.
 */
class EntityForm extends ControllerBase {

  /**
   * Construct a new Composable Entity Info Controller.
   */
  public function __construct(
    protected EntityRepository $entityRepository,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.repository'),
    );
  }

  /**
   * Load the form for a node to edit.
   */
  public function nodeEdit($node_uuid) {
    $storage = "node";
    $display = "composable";
    $node = $this->calculateEntity($storage, $node_uuid);
    $formObject = $this->entityTypeManager()->getFormObject($storage, $display);
    $formObject->setEntity($node);
    $form = $this->formBuilder()->getForm($formObject);
    $menu = [
      [
        'label' => $node->type->entity->label(),
      ],
    ];

    return [
      '#theme' => 'composable_editor__dialog',
      '#form' => $form,
      '#menu' => $menu,
    ];
  }

  /**
   * Function to calculate the entity based on storage and entity value.
   */
  private function calculateEntity($storage, $uuid) {
    $entity = $this->entityRepository->loadEntityByUuid($storage, $uuid);

    if (!$entity) {
      throw new \Exception('Entity not found.');
    }

    return $entity;
  }

}
