<?php

declare(strict_types=1);

namespace Drupal\composable_editor\Form;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The Composable SettingsForm.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Construct a new Settings form.
   *
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityTypeBundleInfo
   *   Drupal entity type bundle service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   Drupal entity field manager.
   */
  public function __construct(
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    protected EntityFieldManagerInterface $entityFieldManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.bundle.info'),
      $container->get('entity_field.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'composable_edit_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['composable_editor.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['basic'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Basic settings:'),
      '#name' => 'basic',
    ];

    $form['settings']['framework'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Framework module name'),
      '#default_value' => $this->config('composable_editor.settings')->get('framework'),
      '#group' => 'basic',
    ];

    $form['settings']['disable_styles'] = [
      '#type' => 'checkbox',
      '#title' => 'Disable default Off-Canvas Styles',
      '#default_value' => boolval($this->config('composable_editor.settings')->get('disable_styles')),
      '#group' => 'basic',
    ];

    $form['settings']['open_load'] = [
      '#type' => 'checkbox',
      '#title' => 'Open Off-Canvas on page load',
      '#default_value' => boolval($this->config('composable_editor.settings')->get('open_load')),
      '#group' => 'basic',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('composable_editor.settings')
      ->set('framework', $form_state->getValue('framework'))
      ->set('disable_styles', boolval($form_state->getValue('disable_styles')))
      ->set('open_load', boolval($form_state->getValue('open_load')))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
