<?php

namespace Drupal\composable_editor_paragraphs\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityRepository;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Controller to edit Paragraphs.
 */
class EntityForm extends ControllerBase {

  /**
   * Construct a new Composable Entity Info Controller.
   */
  public function __construct(
    protected EntityRepository $entityRepository,
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
    );
  }

  /**
   * Load the paragraph add form.
   */
  public function paragraphAdd($node_uuid, $type, $field) {
    $storage = "paragraph";
    $display = "entity_add";
    $node = $this->calculateEntity("node", $node_uuid);
    $paragraph = $this->entityTypeManager()->getStorage('paragraph')->create([
      'type' => $type,
    ]);
    $formObject = $this->entityTypeManager()->getFormObject($storage, $display);
    $formObject->setEntity($paragraph);
    $form = $this->formBuilder()->getForm($formObject, $node, $field);
    $menu = $this->calculateMenuFromParagraph($node, $paragraph);

    return [
      '#theme' => 'composable__dialog',
      '#form' => $form,
      '#menu' => $menu,
    ];
  }

  /**
   * Load the form for a paragraph to edit.
   */
  public function paragraphEdit($node_uuid, $paragraph_uuid) {
    $storage = "paragraph";
    $display = "entity_edit";
    $node = $this->calculateEntity("node", $node_uuid);
    $paragraph = $this->calculateEntity($storage, $paragraph_uuid);
    $formObject = $this->entityTypeManager()->getFormObject($storage, $display);
    $formObject->setEntity($paragraph);
    $form = $this->formBuilder()->getForm($formObject, $node);
    $menu = $this->calculateMenuFromParagraph($node, $paragraph);

    return [
      '#theme' => 'composable_editor__dialog',
      '#form' => $form,
      '#menu' => $menu,
    ];
  }

  /**
   * Function to calculate the entity based on storage and entity value.
   */
  private function calculateEntity($storage, $uuid) {
    $entity = $this->entityRepository->loadEntityByUuid($storage, $uuid);

    if (!$entity) {
      throw new \Exception('Entity not found.');
    }

    return $entity;
  }

  /**
   * Function to calculate the menu based on the paragraph entity.
   */
  private function calculateMenuFromParagraph($node, $paragraph, $menu = [], $bundles = []) {
    // Init bundles.
    if (empty($bundles)) {
      $bundles = array_keys($this->entityTypeBundleInfo->getBundleInfo('paragraph'));
    }

    // Add the Paragraph label (last item).
    if (empty($menu)) {
      $menu[] = [
        'label' => $paragraph->type->entity->label(),
      ];
    }

    $parent = $paragraph->getParentEntity();

    if ($parent && in_array($parent->getType(), $bundles)) {
      $menu[] = [
        'label' => $parent->type->entity->label(),
        'url' => "/composable_editor_paragraphs/form/node/" . $node->uuid() . "/paragraph/" . $parent->uuid() . "/edit",
      ];

      return $this->calculateMenuFromParagraph($node, $parent, $menu, $bundles);
    }

    // Add the Node label (first item).
    $menu[] = [
      'label' => $node->type->entity->label(),
      'url' => "/composable_editor/form/node/" . $node->uuid() . "/edit",
    ];

    return array_reverse($menu);
  }

}
