<?php

namespace Drupal\composable_editor_paragraphs;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a helper for paragraph forms.
 */
trait ParagraphFormHelperTrait {

  /**
   * The ParagraphLineageInspector service.
   *
   * @var \Drupal\composable_editor_paragraphs\ParagraphLineageInspector
   */
  private $lineageInspector;

  /**
   * The ParagraphLineageRevisioner service.
   *
   * @var \Drupal\composable_editor_paragraphs\ParagraphLineageRevisioner
   */
  private $lineageRevisioner;

  /**
   * The Paragraph Node Parent.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface
   */
  protected $rootParent;

  /**
   * Returns the lineage inspector service.
   *
   * @return \Drupal\composable_editor_paragraphs\ParagraphLineageInspector
   *   The lineage inspector service data.
   */
  protected function lineageInspector() {
    if (!isset($this->lineageInspector)) {
      $this->lineageInspector = \Drupal::service('composable_editor_paragraphs.lineage.inspector');
    }
    return $this->lineageInspector;
  }

  /**
   * Returns the lineage revisioner service.
   *
   * @return \Drupal\composable_editor_paragraphs\ParagraphLineageRevisioner|null
   *   The lineage revisioner service data.
   */
  protected function lineageRevisioner() {
    if (!isset($this->lineageRevisioner)) {
      $this->lineageRevisioner = \Drupal::service('composable_editor_paragraphs.lineage.revisioner');
    }
    return $this->lineageRevisioner;
  }

  /**
   * {@inheritdoc}
   *
   * Overridden to store the root parent entity.
   */
  public function buildForm(array $form, FormStateInterface $form_state, EntityInterface $root_parent = NULL) {
    $this->rootParent = $root_parent;

    return parent::buildForm($form, $form_state);
  }

}
